# aalam_common initializer
STATE_STARTED = 1
STATE_RESTARTED = 2
STATE_MIGRATE = 3
STATE_POST_MIGRATE = 4
STATE_VALIDATION = 5  # Called by the packager's validators


CALLBACK_ROUTES = "routes"
CALLBACK_CLEANUP = "cleanup"
CALLBACK_MIGRATE = "migrate"
CALLBACK_MIGRATE_COMPLETED = "migrate_completed"



# coding=utf8
import logging
import os
import webob
import six
import yaml
import aalam_common.auth as zauth
import aalam_common.exceptions as zexc
from aalam_common.redisdb import redis_conn, baseapp_redisify_name
import aalam_common.utils as zutils
import aalam_common.wsgi as wsgi
from aalam_common.config import cfg


yaml_dict = {}


# Following are permission map rules
# all:
#    Need all permissions to access the url
# any:
#    Need atleast one of permissions to access the url
# deny_anon:
#    Cannot be accessed by anonymous users
# deny_ext:
#    Allowed only from the apps running internally
# deny_exc(provide/app,...):
#    Deny all except the apps in the args


class Permissions(object):
    ALL_APPS = "all_apps"
    ALL_REMOTE = "all_remote"

    def __init__(self, condition=None, permissions=[]):
        self.condition = condition
        self.permissions = permissions
        self._deny_anon = False
        self._deny_ext = False
        self._deny_exc = []

    def __call__(self):
        # This is important, otherwise python-routes
        # will put this object as unicode in its route args
        pass

    @staticmethod
    def all(*args):
        return Permissions("all", args)

    @staticmethod
    def any(*args):
        return Permissions("any", args)

    def deny_anon(self):
        self._deny_anon = True
        return self

    def deny_ext(self):
        self._deny_ext = True
        return self

    def deny_exc(self, *args):
        self._deny_exc = args
        return self

    def check(self, request):
        if self._deny_anon and zauth.is_anonymous_user(request):
            raise webob.exc.HTTPForbidden()

        if self._deny_ext and request.auth.get("external", None):
            raise webob.exc.HTTPForbidden()

        if self._deny_exc:
            if Permissions.ALL_REMOTE in self._deny_exc and \
                not request.auth.get('remote_id', None):
                raise webob.exc.HTTPForbidden()

            if not request.auth.get("internal", None):
                raise webob.exc.HTTPForbidden()

            if Permissions.ALL_APPS not in self._deny_exc and \
                    request.auth.get("from", None) not in self._deny_exc:
                raise webob.exc.HTTPForbidden()
            else:
                # We don't need the permissions because we are allowed by
                # choice of the app
                return

        suser_perms = set(request.user_perms)
        found = False
        for p in self.permissions:
            perm_fq = frame_permission(cfg.CONF.app_provider_code,
                                       cfg.CONF.app_code,
                                       *p.split('/'))
            id = RoleCache.get(perm_fq)
            if id not in suser_perms and self.condition == 'all':
                msg = "You do not have enough permissions"
                raise webob.exc.HTTPForbidden(explanation=msg)
            elif id in suser_perms and self.condition == 'any':
                # atlest one is there, break now
                found = True
                break

        if self.condition == "any" and not found:
            raise webob.exc.HTTPForbidden()


def parse_yaml_dict(permissions_map):
    if not os.path.exists(permissions_map):
        yaml_dict = {}
        return yaml_dict

    with open(permissions_map, "r") as stream:
        yaml_dict = yaml.load(stream)
    return yaml_dict


def get_yaml_dict(permissions_map):
    global yaml_dict
    if yaml_dict:
        return yaml_dict

    yaml_dict = parse_yaml_dict(permissions_map)
    return yaml_dict


def get_permission_groups(permissions_map):
    if not os.path.exists(permissions_map):
        return []

    with open(permissions_map, "r") as stream:
        yaml_dict = yaml.load(stream)

    return [group for group, _ in yaml_dict['permission-groups'].items()]


def frame_permission(provider, app, perm_group, perm):
    return "/".join([provider, app, perm_group, perm])


def break_permission(permission):
    try:
        (provider, app, perm_group, perm) = permission.split("/")
        return (provider, app, perm_group, perm)
    except ValueError:
        raise zexc.InvalidPermissionFormat()


class RoleCache(object):
    _cache = {}

    @staticmethod
    def get(permission):
        (provider, app, grp, perm) = break_permission(permission)
        value = RoleCache._cache.get(permission, None)
        if value:
            return value

        logging.info("Caching permissions of group %s" % grp)
        # the cache needs to be updated
        resp = zutils.request_local_server(
            "GET", "/aalam/base/permissions/%s/%s/%s" % (provider, app, grp))
        if resp.status_code == 404:
            logging.error("Unable to find permission group")
            return None

        if resp.status_code != 200:
            logging.error("Unable to get permissions for group %s, because %s"
                          % (grp, resp.text))
            raise zexc.UnableToUpdateRoleCache(grp, resp.status_code,
                                               resp.text)

        data = resp.json()
        ret_value = None
        for perm_obj in data:
            fq_permission = frame_permission(
                provider, app, grp, perm_obj['name'])
            RoleCache._cache[fq_permission] = perm_obj['id']
            if fq_permission == permission:
                ret_value = perm_obj['id']

        return ret_value


def is_user_admin(inp):
    if inp is None:
        return False

    is_str = isinstance(inp, six.string_types) or \
        isinstance(inp, six.text_type)
    if is_str:
        user_val = inp
    elif isinstance(inp, wsgi.AALAMRequest):
        (_, user_val) = zauth.get_auth_user_id(inp, deny_anon=False)

    return user_val in zutils.get_common_rkey('adminusers', 'smembers')


class RolesMgmt(wsgi.Middleware):
    def __init__(self, app):
        self.ydict = get_yaml_dict(cfg.CONF.permissions_map)
        super(RolesMgmt, self).__init__(app)

    def pre(self, request):
        (_, user_val) = zauth.get_auth_user_id(request, deny_anon=False)
        if user_val and is_user_admin(user_val) or not user_val:
            return
        from_internal = zauth.is_auth_internal(request)
        if from_internal and from_internal == "aalam/base":
            # permit all base requests
            return

        route = request.environ['wsgiorg.routing_args'][1]
        url_perms = route.get('permissions', None)

        user_perms = []
        if user_val:
            rkey = baseapp_redisify_name("permissions-%s" % user_val)
            user_perms = zutils.get_common_rkey(rkey, 'smembers')
            if not user_perms:
                logging.info("Permissions for %s is not cached yet" % user_val)
                resp = zutils.request_local_server(
                    "POST",
                    "/aalam/base/permissions/cache/%s"
                    % user_val)
                if resp.status_code not in [200, 404]:
                    logging.error("Unable to cache the user information")
                    raise webob.exc.HTTPInternalServerError()
                user_perms = zutils.get_common_rkey(rkey, 'smembers')

        user_perms = [int(x) for x in user_perms]
        if not user_perms and not from_internal:
            # This should be a valid user account but has not permissions
            # configured in this account
            request.auth = {}
        request.user_perms = user_perms

        if url_perms is None:
            # permission not configured for this method
            return None

        url_perms.check(request)


def get_id_from_rolecache(permission):
    import webob
    try:
        perm_id = RoleCache.get(permission)
        if perm_id is None:
            msg = "Permission(%s) not found" % permission
            raise webob.exc.HTTPNotFound(explanation=msg)
    except zexc.UnableToUpdateRoleCache:
        raise webob.exc.HTTPInternalServerError()
    except zexc.InvalidPermissionFormat:
        msg = "Permission(%s) is not formatted properly" % permission
        raise webob.exc.HTTPBadRequest(explanation=msg)

    return perm_id


def get_id(grp, name):
    permission = frame_permission(
        cfg.CONF.app_provider_code, cfg.CONF.app_code, grp, name)
    return get_id_from_rolecache(permission)


def is_client_authorized(request, grp, name):
    if request.user_perms:
        return get_id(grp, name) in request.user_perms
    else:
        return False



import importlib
import json
import re
import os
import logging
import webob
import yaml
from aalam_common import wsgi, auth as zauth
from aalam_common.config import cfg
from aalam_common import utils as zutils


HTTP_NO_ACTION_NEEDED = 204


class Hooks(wsgi.Middleware):
    def __init__(self, app):
        self._hooks_map = dict()
        self._hooked_map = dict()
        self._restrict_map = dict()
        self._hooked_regex = None
        self._restrict_regex = None
        self._regex = None
        self._load_map()
        super(Hooks, self).__init__(app)

    def _load_map(self):
        if not getattr(cfg.CONF, "hooks_map", None) or \
                not os.path.exists(cfg.CONF.hooks_map):
            return

        with open(cfg.CONF.hooks_map, "r") as f:
            y = yaml.load(f)
            hooked = y["hook"]
            for hook in hooked:
                self._hooked_map[self._frame_url_key(hook)] = hook.copy()

            try:
                restrict = y['restrict']
                for r in restrict:
                    self._restrict_map[self._frame_url_key(r)] = {
                        "type": r['type'] if 'type' in r else 'BA',
                        "except": r['except'] if 'except' in r else []
                    }
            except KeyError:
                pass

        self._hooked_regex = re.compile(
            "(" + '$)|('.join(self._hooked_map.keys()) + "$)")
        if self._restrict_map:
            self._restrict_regex = re.compile(
                "(" + "$)|(".join(self._restrict_map.keys()) + "$)")

    def _regexify_url(self, url):
        return url.replace("*", "[^\/]*?")

    def _frame_url_key(self, dictionary):
        return " ".join([dictionary['method'].upper(),
                         self._regexify_url(dictionary['url'])])

    def add_hooks(self, request):
        try:
            hooks = request.json
        except Exception:
            raise webob.exc.HTTPBadRequest(
                explanation="Input content not proper")

        logging.debug("Adding %d hooks" % len(hooks))
        for h in hooks:
            url_key = self._frame_url_key(h)
            final_type = ""
            for t in list(h['type']):
                if self._is_restricted(h['url'], h['method'],
                                       t, h['hooker']):
                    # This url is restricted
                    continue
                else:
                    final_type += t

            if not final_type:
                continue
            else:
                h['type'] = final_type

            if url_key in self._hooks_map:
                exists = False
                for i in self._hooks_map[url_key]:
                    if i['url'] == h['url'] and \
                            i['method'] == h['method'] and \
                            i['hooker'] == h['hooker'] and \
                            i['type'] == h['type']:
                        exists = True
                        break
                if not exists:
                    self._hooks_map[url_key].append(h.copy())
            else:
                self._hooks_map[url_key] = [h.copy()]

        self._regex = re.compile(
            "(" + '$)|('.join(self._hooks_map.keys()) + "$)")
        return webob.response.Response(status=200)

    def log_hooks(self, request):
        # This is just for logging purposes.
        logging.debug("Hooks map %s" % self._hooks_map)
        logging.debug("Hooks regex %s" % (self._regex and self._regex.pattern))

    def delete_hook(self, request):
        try:
            hooks = request.json
        except Exception:
            raise webob.exc.HTTPBadRequest(
                explanation="Input content not proper")

        logging.debug("Removing %d hooks" % len(hooks))
        for h in hooks:
            url_key = self._frame_url_key(h)
            if url_key not in self._hooks_map:
                return webob.response.Response(status=200)
            self._hooks_map[url_key] = [
                x for x in self._hooks_map[url_key] if not
                (h['type'] == x['type'] and
                 h['hooker'] == x['hooker'])]
        self._regex = re.compile(
            "(" + '$)|('.join(self._hooks_map.keys()) + "$)")
        return webob.response.Response(status=200)

    def _import(self, name):
        mod = __import__(name)
        components = name.split('.')
        for comp in components[1:]:
            mod = getattr(mod, comp)
        return mod

    def _call_callback(self, cb, request):
        (module, method) = cb.split(":")
        try:
            mod = importlib.import_module(module)
            meth = getattr(mod, method)
        except Exception as e:
            # Unable to call the callback - return no action
            return webob.Response(
                status=HTTP_NO_ACTION_NEEDED)

        # the call back method should do the following
        # - return None => Implies, No action need from this hook
        # - return a jsonable object => Implies, send this data to the
        #                               end client before processing
        #                               the main URL
        # - raise an Exception => Implies return this error to client
        #                         The exception object should have
        #                         an attribute named 'status_code'
        try:
            out = meth(request)
        except Exception as e:
            import traceback
            traceback.print_exc()
            status_code = getattr(e, 'status_code', HTTP_NO_ACTION_NEEDED)
            body = getattr(e, 'body', None)
            return webob.Response(
                status=status_code, body=body)

        status_code = HTTP_NO_ACTION_NEEDED
        body = None
        if out:
            try:
                body = json.dumps(out)
                status_code = 200
            except Exception as e:
                pass
        return webob.Response(
            status=status_code, body=body)

    def hook_callback(self, request):
        params = request.params.copy()
        hooked_url_key = self._match_hook_url_key(
            params['method'], params['url'],
            self._hooked_regex, self._hooked_map)
        if not hooked_url_key:
            raise webob.exc.HTTPNotFound()

        cb = self._hooked_map[hooked_url_key]['handler']
        return self._call_callback(cb, request)

    def _match_hook_url_key(self, method, path, regex, hmap):
        url_key = ' '.join([method, path])
        m = re.match(regex, url_key)
        if not m:
            return
        index = next(i for i, j in enumerate(m.groups()) if j)
        return list(hmap.keys())[index]

    def _is_restricted(self, method, path, before_after, hooker):
        if self._restrict_regex:
            restrict_url_key = self._match_hook_url_key(
                method, path, self._restrict_regex, self._restrict_map)
            if restrict_url_key:
                r = self._restrict_map[restrict_url_key]
                if before_after in r['type'] and \
                        hooker not in r['except']:
                    # restrict hooking
                    return True

        return False

    def process_hooks_b(self, request, avoid_cb_of_app):
        if not self._regex:
            # No hooks registered
            return

        hook_url_key = self._match_hook_url_key(
            request.method, request.path, self._regex, self._hooks_map)
        if not hook_url_key:
            return

        hook_data = {}
        for item in self._hooks_map[hook_url_key]:
            if avoid_cb_of_app and item['hooker'] == avoid_cb_of_app:
                continue
            params = dict(zip(request.params.keys(),
                              request.params.values()))
            headers = dict()
            for k in request.headers:
                headers[k] = request.headers[k]
            if 'B' in item['type']:
                data = {'type': 'B',
                        'headers': headers,
                        'params': params,
                        'data': request.text}
                kwargs = {'json': data}
                if request.auth and request.auth.get('user'):
                    kwargs['user'] = request.auth.get(
                        'email_id', request.auth.get('mobile', None))
                try:
                    resp = zutils.request_local_server(
                        'POST',
                        "/%s/_/hooks?method=%s&url=%s" % (
                            item['hooker'], request.method,
                            request.path),
                        timeout=0.100,
                        **kwargs)
                    if resp.status_code == 200 and resp.text:
                        hook_data[item['hooker']] = json.loads(resp.text)
                except Exception:
                    # probably the timeout exception
                    pass

        # give the pre-hooks data to the API for processing
        request.hook_data = hook_data

    def process_hooks_a(self, request, response, avoid_cb_of_app):
        if not self._regex:
            # No hooks registered
            return response

        hook_url_key = self._match_hook_url_key(
            request.method,
            request.path,
            self._regex, self._hooks_map)
        if not hook_url_key:
            return response

        # pop the hooks related header from the API response
        hook_data = response.text
        for item in self._hooks_map[hook_url_key]:
            if avoid_cb_of_app and item['hooker'] == avoid_cb_of_app:
                continue
            if 'A' in item['type']:
                params = dict(zip(request.params.keys(),
                                  request.params.values()))

                data = {'params': params,
                        'type': 'A',
                        'status': response.status_code,
                        'data': hook_data,
                        }
                kwargs = {'json': data}
                if request.auth and request.auth.get('user'):
                    kwargs['user'] = request.auth.get(
                        'email_id', request.auth.get('mobile', None))
                try:
                    zutils.request_local_server(
                        'POST',
                        "/%s/_/hooks?method=%s&url=%s" % (
                            item['hooker'],
                            request.method,
                            request.path),
                        timeout=0.100,
                        **kwargs)
                except Exception:
                    # probably the timeout exception
                    pass
        return response

    def pre(self, request):
        avoid_cb_of_app = None
        if request.path == "/%s/%s/_/hooks" % (
                cfg.CONF.app_provider_code, cfg.CONF.app_code) and \
                zauth.is_auth_internal(request):
            if request.method == "PUT":
                return self.add_hooks(request)
            elif request.method == 'DELETE':
                return self.delete_hook(request)
            elif request.method == 'POST':
                return self.hook_callback(request)
            elif request.method == 'GET':
                return self.log_hooks(request)
            else:
                raise webob.exc.HTTPNotFound(
                    explanation="Invalid method(%s)" % request.method)
        if request.is_nohook:
            avoid_cb_of_app = zauth.is_auth_internal(request)
            if not avoid_cb_of_app:
                # from another source, so, ignore completely
                return

        return self.process_hooks_b(request, avoid_cb_of_app)

    def post(self, request, response):
        avoid_cb_of_app = None
        if request.is_nohook:
            avoid_cb_of_app = zauth.is_auth_internal(request)
            if not avoid_cb_of_app:
                # from another source, so, ignore completely
                return response

        return self.process_hooks_a(request, response, avoid_cb_of_app)



# coding=utf8
from base64 import b64decode
from Crypto.PublicKey import RSA
from Crypto.Signature import PKCS1_PSS
from Crypto.Hash import SHA
from collections import OrderedDict
from datetime import datetime
from ecdsa import VerifyingKey
from ecdsa.util import sigdecode_der
import sys
if sys.version_info[0] == 3:
    import urllib.parse as urllib_parse
else:
    import urllib as urllib_parse

import binascii
import os
import logging
import webob.exc
import aalam_common.wsgi as wsgi
import aalam_common.utils as zutils
from aalam_common.config import cfg


AUTH_COOKIE_NAME = "auth"  # This is the cookie set by central auth server
CUSTOMER_AUTH_COOKIE_NAME = "cauth"
CUSTOMER_AUTH_CURRENT_ID_COOKIE_NAME = "cid"
ANONYMOUS_USERNAME = "Anonymous"
ANONYMOUS_EMAIL = "anonymous"
customer_auth_pubkey = os.path.join(
    os.path.dirname(getattr(cfg.CONF, "pubkey", "/config/keys")),
    "customer_auth.pub.pem")

# a simple key cache to hold the keys so that we don't
# stress the file system
class KeyCache(object):
    def __init__(self, size):
        self._cache_size = size
        self._cache = OrderedDict()

    def get(self, filename):
        if filename not in self._cache:
            with open(filename, "r") as fd:
                self._cache[filename] = fd.read()

        self._cache.move_to_end(filename)
        if len(self._cache) > self._cache_size:
            self._cache.popitem(last=False)
        return self._cache[filename]


key_cache = KeyCache(32)  # a typical key file is 271 bytes


class Auth(wsgi.Middleware):
    def __init__(self, app):
        super(Auth, self).__init__(app)

    def __verify_rsa(self, key_file, message, signature):
        pub_key = RSA.importKey(key_cache.get(key_file))
        auth_signer = PKCS1_PSS.new(pub_key)
        h = SHA.new()
        h.update(message.encode('utf-8'))
        signature = b64decode(signature)

        return auth_signer.verify(h, signature)

    def _handle_token_auth(self, request):
        token = request.headers.get("X-Auth-Token", None)
        if not token:
            return

        try:
            (message, signature) = token.split(";")
        except ValueError:
            raise webob.exc.HTTPUnauthorized()

        (remote_id, _, exp_time) = message.split("#")

        pub_key_file = os.path.join(
            cfg.CONF.auth.userkeys_path,
            "remote-%s.pub" % remote_id)
        if not os.path.exists(pub_key_file):
            raise webob.exc.HTTPUnauthorized()

        if not self.__verify_rsa(pub_key_file, message, signature):
            return webob.exc.HTTPUnauthorized()

        if exp_time:
            expiry = datetime.fromtimestamp(float(exp_time))
            if expiry < zutils.datetime.utcnow():
                logging.warn(
                    "Got an expired token for app %s" % remote_id)
                raise webob.exc.HTTPUnauthorized()

        request.auth = {
            'email_id': "%s@." % remote_id,
            'remote_id': remote_id
        }
        return True

    def _verify_signature(self, key_file, header_value, request):
        (prefix, signature) = header_value.split(";")
        params = request.params
        path = urllib_parse.unquote(request.path)
        if params:
            params = '&'.join(
                ["=".join([k, v]) for k, v in params.items()])
            url = path + "?" + params
        else:
            url = path
        message = "#".join([prefix, url])
        return self.__verify_rsa(key_file, message, signature), prefix

    def _validate_user_signature(self, path, val, ts, signature):
        try:
            vk = VerifyingKey.from_pem(key_cache.get(path))
            cookie_val_b = binascii.a2b_base64(signature)
            message = "#".join([val, ts])
            if (vk.verify(cookie_val_b, message.encode('utf-8'),
                          sigdecode=sigdecode_der)):
                return True
            else:
                raise webob.exc.HTTPUnauthorized()
        except Exception:
            raise webob.exc.HTTPUnauthorized()

    def _fetch_user_keys(self, val):
        payload = {}
        if '@' in val:
            payload['email_id'] = val
        else:
            payload['mobile'] = val
        resp = zutils.request_local_server(
            "POST", "/aalam/base/central/userkey", params=payload)
        if resp.status_code != 200:
            return False

        return True

    def _handle_cookie_auth(self, request):
        auth_cookie = request.cookies.pop(AUTH_COOKIE_NAME, None)
        cauth_cookie = request.cookies.pop(CUSTOMER_AUTH_COOKIE_NAME, None)
        if auth_cookie:
            val, ts, signature = auth_cookie.split('#', 2)
            unq_val = urllib_parse.unquote(val)
            path = os.path.join(cfg.CONF.auth.userkeys_path,
                                "%s.pub" % unq_val)
            if not os.path.exists(path) and \
                    not self._fetch_user_keys(unq_val):
                return False

            if self._validate_user_signature(path, val,
                                             ts, signature):

                request.auth = {"email_id": unq_val} if '@' in unq_val \
                    else {'mobile': unq_val}
                request.auth['user'] = True
                return True

        if cauth_cookie:
            (contact_id, random, signature) = cauth_cookie.split("#", 2)
            message = "#".join([contact_id, random])
            if not self.__verify_rsa(customer_auth_pubkey, message, signature):
                return
            else:
                ids = []
                if ":" in contact_id:
                    ids = contact_id.split(":")
                    curr_id = request.cookies.pop(
                        CUSTOMER_AUTH_CURRENT_ID_COOKIE_NAME, None)
                    contact_id = ids[0] if (
                        not curr_id or curr_id not in ids) else curr_id
                request.auth = {'customer_id': contact_id, 'other_ids': ids}
                return True

        return False

    def _handle_internal_auth(self, request):
        internal = request.headers.get("X-Auth-Internal", None)
        if internal:
            (prefix, signature) = internal.split(";")
            p = prefix.split("/")
            pubkey = os.path.join(
                os.path.dirname(cfg.CONF.pubkey),
                "%s_%s.pub" % (p[0], p[1]))
            (ret, prefix) = self._verify_signature(
                pubkey, internal, request)
            if not ret:
                raise webob.exc.HTTPUnauthorized()

            request.auth = {'internal': True,
                            'from': prefix}

            if len(p) > 2:
                val = p[2]
            else:
                val = get_app_email(p[0], p[1])

            if '@' in val:
                request.auth['email_id'] = val
            else:
                request.auth['mobile'] = val

            return True

        return False

    def _handle_external_auth(self, request):
        signature = request.headers.get('X-Auth-Signature', None)
        if signature:
            prefix = signature[:signature.index(';')]
            ret = False
            if prefix == 'CENTRALPORTAL':
                (ret, _) = self._verify_signature(
                    cfg.CONF.auth.central_pubkey, signature, request)
            elif prefix.startswith('APPSPORTAL'):
                (ret, _) = self._verify_signature(
                    cfg.CONF.auth.apps_server_pubkey, signature, request)
            elif prefix.startswith('BILLINGPORTAL'):
                (ret, _) = self._verify_signature(
                    cfg.CONF.auth.billing_pubkey, signature, request)

            if not ret:
                raise webob.exc.HTTPUnauthorized()

            request.auth = {'external': True}
            if "/" in prefix:
                prefix, val = prefix.split("/")
                params = {'fields': 'id'}
                if '@' in val:
                    params['email'] = val
                else:
                    params['mobile'] = val
                resp = zutils.request_local_server(
                    "GET",
                    "/aalam/base/users",
                    params=params)
                if resp.status_code != 200:
                    raise webob.exc.HTTPUnauthorized()
                data = resp.json()
                user_id = data[0] if data else None

                if '@' in val:
                    request.auth['email_id'] = val
                else:
                    request.auth['mobile'] = val

                request.auth['id'] = user_id

            request.auth['from'] = prefix
            return True

        return False

    def _handle_anonymous(self, request):
        # allow anonymous users, role module will manage it
        request.auth = {"email_id": ANONYMOUS_EMAIL}
        return None

    def pre(self, request):
        if self._handle_token_auth(request):
            pass
        elif self._handle_cookie_auth(request):
            pass
        elif self._handle_internal_auth(request):
            pass
        elif self._handle_external_auth(request):
            pass
        else:
            self._handle_anonymous(request)

        return None


class TestAuth(wsgi.Middleware):
    # This is used just for testing
    def pre(self, request):
        request.auth = {'email_id': 'user@test.test',
                        "internal": True,
                        "from": "aalam/xxxx"}


is_anonymous_user = lambda request: not request.auth or request.auth.get(
    "email_id", None) == ANONYMOUS_EMAIL


def deny_anonymous_user(request):
    if is_anonymous_user(request):
        raise webob.exc.HTTPForbidden(
            explanation="Forbidden for anonymous users")


def get_auth_user_id(request, deny_anon=True):
    auth = request.auth if hasattr(request, "auth") else None
    if is_anonymous_user(request) and deny_anon:
        raise webob.exc.HTTPForbidden(
            explanation="Forbidden for anonymous users")

    if is_anonymous_user(request):
        return (None, ANONYMOUS_EMAIL)

    id = None
    val = auth['email_id'] if 'email_id' in auth else auth.get('mobile', None)

    if 'id' in auth:
        id = auth['id']

    return (id, val)


def get_auth_user(request, deny_anon=True):
    (_, val) = get_auth_user_id(request)
    return val


def is_auth_internal(request):
    auth = request.auth if hasattr(request, "auth") else None
    if auth:
        if auth.get("internal", False):
            return auth.get("from")

    return False


def is_auth_customer(request):
    auth = request.auth if hasattr(request, "auth") else None
    if auth:
        return auth.get("customer_id", False)

    return False


def is_auth_external(request):
    auth = request.auth if hasattr(request, "auth") else None
    if auth:
        if auth.get("external", False):
            return auth.get("from")

    return False


def is_auth_remote(request):
    auth = request.auth if hasattr(request, 'auth') else None
    if auth and auth.get('reomte_id'):
        return auth['remote_id']

    return False


def deny_external_source(request):
    if not is_auth_internal:
        raise webob.exc.HTTPNotFound()


def get_app_email(provider_code, app_code):
    return '_'.join([provider_code, app_code]) + "@%s" % cfg.CONF.hostname


def init_auth():
    pass
